        %%% Trabalho Modelagem 2020 %%%

clc
format
clear
close all
set(groot, 'defaultLegendInterpreter','latex');

%% Definio dos Parmetros Iniciais %%

m1=0.409; % massa da primeira barra (kg)
m2=0.104; % massa da segunda barra (kg)
m3=0.009; % massa da terceira barra - cauda (kg)
a1=0.09375; % distncia do baricentro da primeira barra (m)
a2=0.0255; % distncia do baricentro da segunda barra (m)
a3=0.015; % distncia do baricentro da terceira barra (m)
l1=0.1875; % comprimento da primeira barra (m)
l2=0.0625; % comprimento da segunda barra (m)
l3=0.03; % comprimento da terceira barra (m)
ro=998; % densidade da gua (kg/m)
L=0.075; % Corda da cauda (m)
C_c=0.03/2; % Semi-comprimento da cauda (m)
pi_t=pi; % pi
c1=3e-4; % Coeficiente de amortecimento da primeira unio (N*m*s/rad)
c2=1e-4; % Coeficiente de amortecimento da segunda unio (N*m*s/rad)
k1=4.91e-2; % Coeficiente elstico da primeira unio (N*m/rad)
k2=3.54e-3; % Coeficiente elstico da segunda unio (N*m/rad)
Um=0.3; % Velocidade do escoamento (m/s)
Cd=0.5; % Coeficiente de arrasto hidrodinmico
S=7.069e-4; % rea total do peixe (m)
Fd=0.5*Cd*ro*Um^2*S; % Fora de arrasto hidrodinmico (N)

%% Definio das Equaes de Movimento no Lineares
%Definio dos vetores
h=0.05; % step
t_f=(80+0.05);
t=0:h:t_f; % vetor tempo
x_2_pontos=zeros(size(t)); % vetor da acelerao em x
x_ponto=zeros(size(t)); % vetor da velocidade em x
x=zeros(size(t)); % vetor da posio em x
theta2_2_pontos=zeros(size(t)); % vetor da acelerao em theta2
theta2_ponto=zeros(size(t)); % vetor da velocidade em theta2
theta2=zeros(size(t)); % vetor da posio em theta2
theta3_2_pontos=zeros(size(t)); % vetor da acelerao em theta3
theta3_ponto=zeros(size(t)); % vetor da velocidade em theta3
theta3=zeros(size(t)); % vetor da posio em theta3
X=[x_ponto; x; theta2_ponto; theta2; theta3_ponto; theta3];

%Vetor de entradas - variaes para simulao de diferentes entradas
    %step
    %T_theta=-ones(size(t));
    %T_theta(1)=0;
    %T=ones(size(t));
    %T(1)=0;
    %Fd=ones(size(t))*Fd;
    %Fd(1)=0;
    
    %impulse
    T_theta=zeros(size(t));
    %T_theta(1)=1/h;
    T=zeros(size(t));
    %T(1)=1/h;
    Fd=zeros(size(t));
    %Fd(1)=1/h;
    
    %ramp
    %T_theta=t;
    %T=t;
    %Fd=t;
    
    %senoidal
    %T_theta=sin(t);
    %T=0.35*sin(0.4*t);
    %Fd=sin(t);
    
u = [T_theta; T; Fd];
ut = t;

%Definio dos valores iniciais
x(1)=0;
x_ponto(1)=0;
theta2(1)=pi_t/18;
theta2_ponto(1)=-0.02;
theta3(1)=pi_t/12;
theta3_ponto(1)=-0.02;
X0 = [x_ponto(1), x(1), theta2_ponto(1), theta2(1), theta3_ponto(1), theta3(1)]; %vetor com condies iniciais

%Variveis auxiliares
J2=m2*a2^2;
J3=m3*a3^2;
M11=J2+m2*a2^2+m3*l2^2+pi_t*ro*L*C_c^2*l2^2;
M12=m3*a3*l2+pi_t*ro*L*C_c^2*l2*a3;
M21=m3*a3*l2+pi_t*ro*L*C_c^2*l2*a3;
M22=J3+m3*a3^2+pi_t*ro*L*C_c^2*a3^2;
C_11=c1+c2+2*pi_t*ro*L*C_c*Um*l2^2;
C_12=-c2+pi_t*ro*L*C_c*Um*(2*a3-C_c*Um)*l2;
C_21=-c2+2*pi_t*ro*L*C_c*Um*l2*a3;
C_22=c2+pi_t*ro*L*C_c*Um*(2*a3-C_c*Um)*a3;
K_11=k1+k2;
K_12=-k2+2*pi_t*ro*L*C_c*Um^2*l2;
K_21=-k2;
K_22=k2+2*pi_t*ro*L*C_c*Um^2*a3;
alfa=pi_t*ro*L*C_c^2;
C22=(-M22*M11*C_11+M12*M11*C_21)/(M22*M11-M21*M12);
C23=(-M22*M11*C_12+M12*M11*C_22)/(M22*M11-M21*M12);
K22=(-M22*M11*K_11+M12*M11*K_21)/(M22*M11-M21*M12);
K23=(-M22*M11*K_12+M12*M11*K_22)/(M22*M11-M21*M12);
C32=(M21*C_11/M11-C_21);
C33=(M21*C_12/M11-C_22);
K32=(M21*K_11/M11-K_21);
K33=(M21*K_12/M11-K_22);
beta=(M22*M11-2*M21*M12)/(M22*M11-M21*M12);

%Integrao com ODE45
tspan=t;
[tode,y] = ode45(@ (t,y) funcao_derivada(y,u,ut,beta,pi_t,l2,a3,m1,m2,m3,Cd,S,C_c,L,ro,Um,M11,M12,M21,M22,C22,C23,C32,C33,K22,K23,K32,K33,t),tspan,X0);
Xode = y';
tode = tode';

%Avaliao grfica linear pelo comando ode45
figure()
hold on
plot(tode,Xode(1,:),'LineWidth',2);
plot(tode,Xode(2,:),'LineWidth',2);
hold off
legend('$\dot{x}_G$ (m/s)','$x_G$ (m)');
xlabel('Tempo (s)')
ylabel('Amplitude')
trocar();
grid on;
grid minor

figure()
hold on
plot(tode,Xode(3,:),'LineWidth',2);
plot(tode,Xode(4,:),'LineWidth',2);
plot(tode,Xode(5,:),'LineWidth',2);
plot(tode,Xode(6,:),'LineWidth',2);
hold off
legend('$\dot{\theta}_2$ (rad/s)','$\theta_2$ (rad)','$\dot{\theta}_3$ (rad/s)','$\theta_3$ (rad)');
xlabel('Tempo (s)')
ylabel('Amplitude')
trocar();
grid on;
grid minor

%% Definio do Espao de Estados com Equaes Linearizadas

M=[m1+m2+m3, 0,  0, 0,       0,           0;
      0,     1,  0, 0,       0,           0;
      0,     0, M11,0,       0,           0;
      0,     0,  0, 1,       0,           0;
      0,     0,  0, 0, M22-(M12*M21/M11), 0; 
      0,     0,  0, 0,       0,           1];
 
A=[ -Cd*ro*Um*S, 0,  0,   0,   0,   0;
        1,       0,  0,   0,   0,   0;
        0,       0, C22, K22, C23, K23;
        0,       0,  1,   0,   0,   0;
        0,       0, C32, K32, C33, K33;
        0,       0,  0,   0,   1,   0];

B=[alfa*l2*beta/M11+alfa*a3*M21/(M22*M11-M12*M21), 0,   1;
                       0,                          0,   0;
                       0,                        beta,  0;
                       0,                          0,   0;
                       0,                       M21/M11,0;
                       0,                          0,   0];
C=eye(6);
D = zeros(6,3);

sys_ss = ss(A,B,C,D);

%Polinmio caracterstico
P_c = poly(A);

% Autovalores
autovalores = eig(A);

%% Avaliao do comportamento no domnio da frequncia
%Obteno das funes de transferncia
sys_tf = tf(sys_ss);
    
%Obteno dos valores de polos, zeros e ganhos
[z,p,k] = zpkdata(sys_ss);
pole(sys_ss)

%Avaliao grfica dos polos do sistema
figure
pzmap(sys_ss)

%Obteno das frequncias naturais e dos coeficientes de amortecimento
damp(sys_tf)

%Diagramas de bode
figure
bode(sys_ss)
xlim([0.001 1])

%% Avaliao do comportamento no domnio do tempo
%Avaliao grfica da resposta do sistema ao impulso
figure
impulse(sys_ss(1,1))

%Avaliao grfica da resposta do sistema ao step
% figure
% step(sys_ss)

%Integrao com lsim
Xlsim = lsim(sys_ss,u,t,X0);
Xlsim = Xlsim';

%Avaliao grfica linear pelo comando lsim
figure();
hold on
plot(t,Xlsim(1,:),'LineWidth',2);
plot(t,Xlsim(2,:),'LineWidth',2);
hold off
legend('$\dot{x}_G$ (m/s)','$x_G$ (m)');
xlabel('Tempo (s)')
ylabel('Amplitude')
trocar();
grid on;
grid minor

%Avaliao grfica angular pelo comando lsim
figure();
hold on
plot(t,Xlsim(3,:),'LineWidth',2);
plot(t,Xlsim(4,:),'LineWidth',2);
plot(t,Xlsim(5,:),'LineWidth',2);
plot(t,Xlsim(6,:),'LineWidth',2);
hold off
legend('$\dot{\theta}_2$ (rad/s)','$\theta_2$ (rad)','$\dot{\theta}_3$ (rad/s)','$\theta_3$ (rad)');
xlabel('Tempo (s)')
ylabel('Amplitude')
trocar();
grid on;
grid minor

%Integrao por matriz de transio e termos forantes
max=150;

phi = zeros(size(A));
for i=1:max
    aux = (A^(i-1))*(h^(i-1))/factorial(i-1);
    phi = phi+aux;
end

gamma = zeros(size(A));
for i=1:max
    aux = (A^(i-1))*(h^(i-1))/factorial(i);
    gamma = gamma+aux;  
end
gamma=h*gamma;

Xmt(:,1)=X0';
for j=1:(size(t)-1)
    Xmt(:,j+1) = phi*Xmt(:,j)+gamma*B*u(:,j);
end

%Avaliao grfica do sistema por matriz de transio
figure();
hold on
plot(t,Xmt(1,:),'LineWidth',2);
plot(t,Xmt(2,:),'LineWidth',2);
hold off
legend('$\dot{x}_G$ (m/s)','$x_G$ (m)');
xlabel('Tempo (s)')
ylabel('Amplitude')
trocar();
grid on;
grid minor

%% Animao grfica da resposta
Xv=Xmt;
%Xv=Xode;

figure
xA = -((l1-a1)*(m2+m3)+(m2*a2+m3*l3)*cos(Xv(4,:))+m3*a3*cos(Xv(6,:)))/(m1+m2+m3)+Xv(2,:)-a1;
yA = zeros(1,length(xA));
xB = xA+l1;
yB = yA;
xC = xB+l2*cos(Xv(4,:));
yC = l2*sin(Xv(4,:));
xD = xC+l3*cos(Xv(6,:));
yD = yC+l3*sin(Xv(6,:));
n=length(xA);
v = VideoWriter('C:\Users\Usuario\Google Drive\Modelagem\videos\GIF.avi');
v.Quality = 95;
v.FrameRate = 1/h;
open(v);
for i=1:n
Pxa = [ xA(i), xB(i) ];
Pya = [ yA(i), yB(i) ];
Pxb = [ xB(i), xC(i) ];
Pyb = [ yB(i), yC(i) ];
Pxc = [ xC(i), xD(i) ];
Pyc = [ yC(i), yD(i) ];
Pxd = [ xD(i)+0.015*sin(Xv(6,i)), xD(i)-0.015*sin(Xv(6,i))];
Pyd = [ yD(i)-0.015*cos(Xv(6,i)), yD(i)+0.015*cos(Xv(6,i))];
Pxe = [ xD(i)+0.015*sin(Xv(6,i)), xC(i)];
Pye = [ yD(i)-0.015*cos(Xv(6,i)), yC(i)];
Pxf = [ xD(i)-0.015*sin(Xv(6,i)), xC(i)];
Pyf = [ yD(i)+0.015*cos(Xv(6,i)), yC(i)];
Pxg = [ xB(i)+0.02*sin(Xv(4,i)), xC(i)];
Pyg = [ yB(i)-0.02*cos(Xv(4,i)), yC(i)];
Pxh = [ xB(i)-0.02*sin(Xv(4,i)), xC(i)];
Pyh = [ yB(i)+0.02*cos(Xv(4,i)), yC(i)];
Pxi = [ xB(i)+0.02*sin(Xv(4,i)), xA(i)+l1/3];
Pyi = [ yB(i)-0.02*cos(Xv(4,i)), yA(i)-0.02];
Pxj = [ xB(i)-0.02*sin(Xv(4,i)), xA(i)+l1/3];
Pyj = [ yB(i)+0.02*cos(Xv(4,i)), yA(i)+0.02];
Pxk = [ xA(i)+l1/3, xA(i)];
Pyk = [ yA(i)+0.02, yA(i)];
Pxl = [ xA(i)+l1/3, xA(i)];
Pyl = [ yA(i)-0.02, yA(i)];
plot(xA, yA, xA(i),yA(i),'o', xB, yB, xB(i),yB(i),'o', xC, yC, xC(i),yC(i),'o',xD, yD, xD(i),yD(i),'o')
axis([-0.4 0.25 -0.325 0.325])
xlabel('(m)')
ylabel('(m)')
grid on
line(Pxa,Pya)
line(Pxb,Pyb)
line(Pxc,Pyc)
line(Pxd,Pyd)
line(Pxe,Pye)
line(Pxf,Pyf)
line(Pxg,Pyg)
line(Pxh,Pyh)
line(Pxi,Pyi)
line(Pxj,Pyj)
line(Pxk,Pyk)
line(Pxl,Pyl)
trocar()
frame = getframe(gcf);
writeVideo(v,frame);
end
close(v)